#include <avr/io.h>

#define PORT_ADDR PORTA
#define PORT_DATA PORTC
#define DDR_ADDR DDRA
#define DDR_DATA DDRC
#define DDR_CTL DDRD
#define PORT_CTL PORTD

#define PIN_CS PIND7

#define PPIN_OC1A PIND
#define PIN_OC1A PIND5

#define DIR_ALL_OUTPUT 0xFF
#define DIR_ALL_INPUT 0x00

void MOS6581_Init(void)
{
        // Initialize all Ports.
        DDR_ADDR = DIR_ALL_OUTPUT;              // Whole ADDR port as output.
        DDR_DATA = DIR_ALL_OUTPUT;              // Whole DATA port as output.
        DDR_CTL |= (_BV(PIN_CS));               // PORTD7 (/CS) as output.

        PORT_ADDR = 0x00;                       // Set ADDR to 0.
        PORT_DATA = 0x00;                       // Set DATA to 0.

        PORT_CTL |= (_BV(PIN_CS));              // Set /CS to high (MOS
                                                // inactive).

        // Initialize 1MHz Clock on OC1A (PB5 on ATMega32)
        TCNT0 = 0x00;           // Timer/Counter register.
        OCR1A = 0x07;           // Output Compare register.
        TCCR1B = 0x09;          // Timer/Counter control register.
        TCCR1A = 0x40;          // Timer/Counter control register.
}

void MOS6581_WriteByte(uint8_t i_addr, uint8_t i_data)
{
        PORT_ADDR = i_addr;                             // Addr on PORT_ADDR.
        PORT_DATA = i_data;                             // Data on PORT_DATA

        while(!(PPIN_OC1A >> PIN_OC1A) & 0x01){};       // Wait until clock
                                                        // state changes.

        PORT_CTL &= ~(_BV(PIN_CS));                     // Set /CS low
                                                        // (SID active).

        while(PPIN_OC1A & _BV(PIN_OC1A)){};             // Wait until clock
                                                        // state changes.

        PORT_CTL |= (_BV(PIN_CS));                      // Set /CS high
}                                                        // (SID inactive).
